/* file.c -- Copyright (C)  2002-2008 Marco Cesati
 * Mon,  5 May 2003 18:38:28 +0200
 */

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_STDLIB_H
#    include <stdlib.h>
#endif

#include <stdio.h>
#include "gentest.h"

int groups_s[MAXGROUPS] = { 0, };
int groups_t[MAXGROUPS] = { 0, };

int ngroup;
int nquiz;
int nquestion;



/*
    check_field() counts how many occurrences of every token are in
    the ATQ file. In particular, it increments a counter for the
    number of questions, whose address is passed as parameter. It also
    checks that no more than one occurrence exists for any field beside
    "QUESTION", "MULQUESTION", "GOOD", and "BAD". The function returns 1
    in case of error, 0 otherwise.
*/

static int check_field(int ntoken)
{
    static int fcount[MAXTOKEN+1] = {0, };
    
    if (ntoken < 1 || ntoken > MAXTOKEN) {
        fprintf(stderr, "Unexpected token (%d)\n", ntoken);
        return 1;
    }
    
    switch (ntoken) {
    
        case QUESTION:
        case MULQUESTION:
        case ATQFILE:
            return 0;

        default:
            if (!(fcount[ntoken]++)) /* first occurrence is fine */
                return 0;
                
            fprintf(stderr, "\nError in ATQ file: "
                            "%s field duplicated\n", token[ntoken]);
    }
    return 1;
}

/*
    format of the \group token: 
          <group-def> := <number>/<number>
          <group-line> := "\group{"<group-def>[[,<group-def>]...]"}"
    In any group definition, the first number represents the number
    of questions that will be selected among the group, and the second
    number represents the number of questions that build the group.
    The questions of a group are always sequential on the file, thus
    the groups are also defined by the ordering of the questions.
    The function fills the group_s[] and group_t[] arrays, and 
    initializes the counters ngroup, nquiz (the number of questions as seen
    by the student), and nquestions (the total number of questions
    in the file).
*/

static void count_groups(int *items, int numitem)
{
    int i, g;
    
    if (numitem % 2)
        panic("\nError in ATQ file: Wrong format in groups\n"
              "(\\groups{a/b,c/d,e/f,...})\n");
    
    if (numitem > MAXGROUPS * 2)
        panic("\nError in ATQ file: File has too many groups (%d)\n"
              "(Maximum number of groups is %d)\n", numitem/2, MAXGROUPS);

    nquiz = 0;
    nquestion = 0;
    ngroup = numitem / 2;
 
    for (i=g=0; i<numitem; i++, g++) {

        groups_s[g] = items[i];
        groups_t[g] = items[++i];
           
            
        if (groups_s[g] > groups_t[g])
        
            panic("\nError in ATQ file: Group #%d (%d/%d) is not valid\n"
                  "(In a group definition, the second number cannot be smaller"
                  "\nthan the first number)\n", g, groups_s[g], groups_t[g]);

        if (!groups_s[g] || !groups_t[g])
            fprintf(stderr, "\nWARNING: value zero in group definition\n");

        nquiz += groups_s[g];
        nquestion += groups_t[g];        
    }

   for (i=0; i<ngroup; ++i)
        fprintf(stderr, "%d/%d ", groups_s[i], groups_t[i]);
   fprintf(stderr, ")\nEach test sheet will include %d questions choosed among a grandtotal of %d\n",
         nquiz, nquestion);

}


static int good_weights[10] = { 0, };
static int bad_weights[10] =  { 0, };

static void update_weight_stats(int ans)
{
    int tok;
    int *p = (ans == GOOD ? good_weights : bad_weights);
    tok = get_token();
    if (tok != ARG)
        panic("'{' expected after \\bad or \\good\n");
    tok = get_token();
    if (tok != NUM)
        panic("'{<number>}' expected after \\bad{...} or \\good{...}\n");
    if (_value<0||_value>9)
        panic("Answer weight %d out of range {0...9}\n", _value);
    ++p[_value];
    return;
}

static void show_weight_stats(void)
{
    int i, *p;
    fprintf(stderr, "Weights of good answers:");
    p=good_weights;
    for (i=0; i<10; ++i,++p)
        if (*p > 0)
            fprintf(stderr, " %+d(%d)", i, *p);
    fprintf(stderr, ". Weights of bad answers:");
    p=bad_weights;
    for (i=0; i<10; ++i,++p)
        if (*p > 0)
            fprintf(stderr, " %d(%d)", -i, *p);
    fputc('.', stderr);
    fputc('\n', stderr);
}

void readfile(FILE *input)
{
    int tok;
    int cquestion = 0;
 
    restart_atq_scanner(input);

    tok = get_token();
               
    while (tok) {

        switch(tok) {            
            case GOOD:
            case BAD:
                update_weight_stats(tok);
            break;

            case GROUPS:
            {
            
                int i = 0;
                int items[MAXGROUPS*2];

                check_field(tok);
                
                while ((tok = get_token())) {
                    if (tok != NUM)
                        break;
                    items[i++] = _value;
                }
                count_groups(items, i);
                continue;
            }
            
            case QUESTION:
            case MULQUESTION:
                ++cquestion;
                
                /* FALL THROUGH */
            case ATQFILE:
            case ARG:
            case NUM:
            break;
                
            default:
                if (check_field(tok))
                    panic("Aborted at line #%d\n", _linenum);
            break;
        }
        tok = get_token();
    }
    
    if (cquestion != nquestion)
        panic("Error in ATQ file: Wrong number of questions\n"
              "(In your \"%s\" field you specified a total number of "
              "%d questions,\nbut there are %d occurrences of the \"%s\" "
              "and \"%s\" fields)\n", token[GROUPS], nquestion, cquestion, 
              token[QUESTION], token[MULQUESTION]);
              
    if (!nquestion)
        panic("\nCannot find any question in atq file!\n");

#if 0
    fprintf(stderr, "EOF at line %d\n", _linenum);
#endif
    show_weight_stats();
}

